# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

from .parse_common import parse_is_article, parse_is_oa, parse_py_datetime, parse_document_types, parse_fu_tokens
from .parse_categorys import parse_ecoom_categorys
from .parse_country import parse_country, parse_collaboration_type_china, parse_lead_type_china
from .parse_country import parse_collaboration_type, parse_first_country
from .parse_doi import parse_cr_dois
from .parse_tags import parse_tags

from .parse_orcid import parse_orcid_info
from .parse_address import parse_address_info
from .parse_rid import parse_rid_info

ARRAY_KEYS = [
    'document_types',
    'fu_tokens',
    'ecoom_categorys',
    'countrys_c1',
    'countrys_rp',
    'countrys_c1_rp',
    'cr_dois',
    'tags',
]


def add_row_type(item):
    for key in ARRAY_KEYS:
        item[key] = item[key].split(';') if item[key] else []
    for key in item.keys():
        if str(item[key]) == 'nan':
            item[key] = None


def remove_row_type(item):
    for key in ARRAY_KEYS:
        if type(item[key]) is list:
            item[key] = ';'.join([str(i) for i in item[key]])


def parse_version1(items):
    """ version1

    - parser_version
    - is_article
    - is_oa
    - py_datetime
    - ecoom_categorys
    - countrys_c1
    - countrys_rp
    - countrys_c1_rp
    - cr_dois
    - tags
    """

    for row in items:
        row['parser_version'] = 1
        row['document_types'] = parse_document_types(row)
        row['fu_tokens'] = parse_fu_tokens(row)
        row['is_article'] = parse_is_article(row)
        row['is_oa'] = parse_is_oa(row)
        row['py_datetime'] = parse_py_datetime(row)

        row['countrys_c1'] = parse_country(row, field='C1', hmt=True)
        row['countrys_rp'] = parse_country(row, field='RP', hmt=True)
        row['countrys_c1_rp'] = parse_country(row, field='C1', extra_field='RP', hmt=True)
        row['collaboration_type_china'] = parse_collaboration_type_china(row['countrys_c1_rp'])
        row['lead_type_china'] = parse_lead_type_china(row['countrys_c1_rp'])
        row['collaboration_type'] = parse_collaboration_type(row['countrys_c1_rp'])
        row['first_country'] = parse_first_country(row['countrys_c1_rp'])

        row['ecoom_categorys'] = parse_ecoom_categorys(row)
        row['tags'] = parse_tags(row)
        row['cr_dois'] = parse_cr_dois(row)

        row['c1_address_info'] = parse_address_info(row.get('C1', ''))
        row['orcid_info'] = parse_orcid_info(row.get('OI', ''))
        row['rid_info'] = parse_rid_info(row.get('RI', ''))

        add_row_type(row)



# coding: utf-8

import os
import subprocess
from scilib.iterlib import chunks

BASE_DIR = os.path.dirname(__file__)
R_PATH = os.path.join(BASE_DIR, 'gender.R')


def batch_classify(names):
    first_names = [i.split()[0] for i in names]

    output_results = {}
    for chunk in chunks(first_names, size=500):
        output = subprocess.check_output([
            'Rscript',
            R_PATH,
            '--name',
            ','.join(chunk),
        ]).decode('utf-8')
        output_results.update(dict([i.strip().split(',') for i in output.split('\n') if i.strip()]))

    results = []
    for first_name in first_names:
        result = output_results.get(first_name, "unknown")
        if result in ['male']:
            results.append('male')
        elif result in ['female']:
            results.append('female')
        else:
            results.append('unknown')
    return results


def test():
    results = batch_classify(["tom", "lisa", "bob", "test10086"])
    print(results)


if __name__ == '__main__':
    test()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import re


def parse_rid(text):
    """ parse RI field
    """
    if not text or str(text) == 'nan':
        return []
    state = 'NAME'  # NAME | RID
    name = ''
    rid = ''

    results = []
    for c in text:
        if state == 'NAME':
            if c == '/':
                state = 'RID'
                continue
            elif name == '' and c in [' ', ';']:
                continue
            else:
                name += c
        elif state == 'RID':
            if rid.count('-') == 2 and re.search(r'-\d{4}$', rid):
                results.append((name, rid))
                state = 'NAME'
                name = ''
                rid = ''
                continue
            else:
                rid += c
                continue
        else:
            raise ValueError(state)

    if name and rid:
        results.append((name, rid))
    return results


def parse_rid_info(text):
    results = parse_rid(text)
    infos = []
    for name, rid in results:
        infos.append(dict(
            name=name,
            rid=rid,
        ))
    return infos



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

from .base import call, call_batch, call_from_file


def start_with_cd(working_dir):
    return call_batch(
        call('clear all'),
        call('macro drop _all'),
        call(f'cd "{working_dir}"'),
        call('pwd'),
    )


def xls2dta(xlsx_file, dta_file):
    return call_batch(
        call(f'capture erase "{dta_file}"'),
        call(f'xls2dta:import excel using "{xlsx_file}", firstrow'),
        call(f'use "{dta_file}"')
    )


def summary(var_list, row_list='变量名称 均值 标准差 最大值 方差 最小值', word_file='mytable.docx'):
    return call_from_file(
        '_summary.do',
        REPLACE_VAR_LIST=var_list,
        REPLACE_ROW_LIST=row_list,
        REPLACE_WORE_FILE=word_file,
    )


def reg(var_list):
    return call_batch(
        call('reg', var_list),
        call('estat vif'),
    )


def nbreg(var_list, word_file='mytable.docx'):
    return call_batch(
        call('nbreg', var_list, ',r'),
        call('est store m1'),
        call("""
            reg2docx m1 using REPLACE_WORE_FILE,         ///
            b(%5.3f) t(%5.3f) scalars(N p(%9.3f))   ///
            title("表1: 负二项回归输出") mtitles("模型") append
        """.replace('REPLACE_WORE_FILE', word_file)),
        call('reg', var_list),
        call('estat imtest, white'),
        call('reg', var_list, ', vce(robust)'),
        call('est store m2'),
        call("""
            reg2docx m2 using REPLACE_WORE_FILE,         ///
            b(%5.3f) se(%9.2f) scalars(N p(%9.3f))   ///
            title("表2: 线性回归模型") mtitles("模型") append
        """.replace('REPLACE_WORE_FILE', word_file)),
    )



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import json
import pandas as pd
from optparse import OptionParser
from collections import Counter

A_TO_Z = [chr(i) for i in range(65, 65 + 26)]
A_TO_Z_RANGE = [(a, A_TO_Z[i + 1]) for i, a in enumerate(A_TO_Z[:-1])]


def make_nodes_edges(items):
    nodes = set()
    edges = []
    for item in items:
        for a, b in A_TO_Z_RANGE:
            if item.get(a) and item.get(b):
                nodes.add(item.get(a))
                nodes.add(item.get(b))
                edges.append((item.get(a), item.get(b)))
    nodes = list(nodes)
    return nodes, edges


def make_matrix_csv(nodes, edges):
    lines = []
    for nodea in nodes:
        line = [nodea]
        for nodeb in nodes:
            count = len([True for a, b in edges if a == nodea and b == nodeb])
            line.append(count)
        lines.append(','.join([str(i) for i in line]))
    return '\n'.join(lines)


def make_sankey_data(nodes, edges):
    data_nodes = [dict(name=node) for node in nodes]
    counter = Counter(edges)
    data_links = []
    for (a, b), count in counter.most_common():
        if a == b:
            continue
        data_links.append(dict(
            source=nodes.index(a),
            target=nodes.index(b),
            value=count,
        ))
    return dict(
        nodes=data_nodes,
        links=data_links,
    )


def make_arc_data(nodes, edges):
    data_nodes = []
    edges_unique = [edge for edge in edges if edge[0] != edge[1]]
    for index, node in enumerate(nodes):
        data_nodes.append(dict(
            id=index,
            name=node,
            value=len([True for edge in edges_unique if edge[1] == node]),
        ))
    data_links = []
    for a, b in edges_unique:
        data_links.append(dict(
            source=nodes.index(a),
            target=nodes.index(b),
        ))
    return dict(
        nodes=data_nodes,
        links=data_links,
    )


def make_blance_analytics(nodes, edges):
    edges_unique = [edge for edge in edges if edge[0] != edge[1]]
    values = []
    for node in nodes:
        in_value = len([True for a, b in edges_unique if b == node])
        out_value = len([True for a, b in edges_unique if a == node])
        value = in_value - out_value
        values.append(dict(
            name=node,
            in_value=in_value,
            out_value=out_value,
            value=value,
        ))
    return values


def from_excel(path):
    df = pd.read_excel(path)
    return [dict({k: v for k, v in row.items() if str(v) != 'nan'}) for i, row in df.iterrows()]


def run(from_path, to_path, to_type):
    items = from_excel(from_path)
    nodes, edges = make_nodes_edges(items)
    if to_type == 'sankey_json':
        data = make_sankey_data(nodes, edges)
        with open(to_path, 'w') as f:
            json.dump(data, f, indent=2, sort_keys=True, ensure_ascii=False)
    elif to_type == 'arc_json':
        data = make_arc_data(nodes, edges)
        with open(to_path, 'w') as f:
            json.dump(data, f, indent=2, sort_keys=True, ensure_ascii=False)
    elif to_type == 'edges_xlsx':
        df = pd.DataFrame.from_records([dict(source=a, target=b) for a, b in edges if a != b])
        df.to_excel(to_path)
    elif to_type == 'blance_analytics':
        df = pd.DataFrame.from_records(make_blance_analytics(nodes, edges))
        df.to_excel(to_path)
    elif to_type == 'vosviewer':
        data = make_matrix_csv(nodes, edges)
        with open(to_path, 'w') as f:
            f.write(data)


def main():
    parser = OptionParser()
    parser.add_option("--from", action="store", type="str", dest="from_path")
    parser.add_option("--to", action="store", type="str", dest="to_path")
    parser.add_option("--to_type", action="store", type="str", dest="to_type", default="sankey_json")
    options, args = parser.parse_args()
    if not (options.from_path and options.to_path):
        parser.print_help()
        return
    run(options.from_path, options.to_path, options.to_type)


if __name__ == '__main__':
    main()



# coding: utf-8

import os
import json
import uuid
import shutil
import zipfile

TEMP_PATH = '/tmp'


def mkdirp(path):
    """ mkdirp
    """
    if not os.path.exists(path):
        os.mkdir(path)


def write_file(path, content):
    """ write_file
    """
    with open(path, 'w') as f:
        f.write(content)


def read_file(path):
    with open(path, 'r') as f:
        return f.read()


def replace_file_content(path, source, target):
    content = read_file(path)
    write_file(path, content.replace(source, target))


def unshift_file_content(path, append_content):
    content = read_file(path)
    write_file(path, append_content + content)


def append_file_content(path, append_content):
    content = read_file(path)
    write_file(path, content + append_content)


def read_json_file(path):
    with open(path, 'r') as f:
        return json.load(f)


def create_temp_dir():
    dir_name = os.path.join(TEMP_PATH, str(uuid.uuid4()))
    os.mkdir(dir_name)
    return dir_name


def remove_temp_dir(dir_path):
    if dir_path.startswith(TEMP_PATH) and os.path.isdir(dir_path):
        shutil.rmtree(dir_path)
    else:
        raise Exception(dir_path)


def unzip(path, target):
    fp = zipfile.ZipFile(path, 'r')
    fp.extractall(target)
    fp.close()


def text_to_temp_file(text, ext='.txt'):
    file_name = os.path.join(TEMP_PATH, str(uuid.uuid4()) + ext)
    with open(file_name, 'w') as f:
        f.write(text)
    return file_name


def get_dir_size(start_path='.'):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            total_size += (os.path.getsize(fp) if os.path.isfile(fp) else 0)
    return total_size



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import asyncio
import pandas as pd
from pathlib import Path
import concurrent.futures


def _read_text_format_lines(lines):
    items = []
    current_item = None
    current_item_key = None
    for index, line in enumerate(lines):
        if line.startswith('null') and line.endswith('null'):
            continue
        start = line[:2]
        end = line[3:]
        if start in ['FN', 'VR', 'EF']:
            continue
        elif start == 'PT' or 'nullPT' in line:
            current_item = {}
            current_item[start] = [end]
            current_item_key = start
        elif start == '  ':
            if current_item and current_item_key:
                current_item[current_item_key].append(end)
        elif start == 'ER':
            items.append(current_item)
            current_item = None
            current_item_key = None
        elif len(start) == 2:
            if current_item is None:
                print('[WARN] _read_text_format_lines', index, line)
            current_item[start] = [end]
            current_item_key = start
        else:
            continue
    return items


def read_text_format_path(path, *, export_type='other_text'):
    """ Read data from path

    export_type: other_text | fast5k_csv
    """
    if export_type == 'fast5k_csv':
        df = pd.read_csv(path, sep='\t', index_col=False)
        return [dict(row) for i, row in df.iterrows()]

    with open(path, 'r', encoding='utf-8-sig') as f:
        lines = f.read().split('\n')
    try:
        items = _read_text_format_lines(lines)
    except Exception:
        print('[WARN] read_text_format_dir', path)
        raise

    data_list = []
    for item in items:
        data = {}
        for k, v in item.items():
            if k in ['WC', 'SC']:
                data[k] = ' '.join(v)
            else:
                data[k] = '; '.join(v)
        data_list.append(data)
    return data_list


def scan_text_format_dir(abs_path, *, globs=None):
    globs = globs or ['**/*.txt', '**/*.csv', '*.csv', '*.txt']
    for glob in globs:
        for path in Path(abs_path).glob(glob):
            yield path


def read_text_format_dir(abs_path, *, globs=None, export_type='other_text'):
    for path in scan_text_format_dir(abs_path, globs=globs):
        for item in read_text_format_path(path, export_type=export_type):
            yield item


def read_text_format_dir_as_pd(abs_path, *, globs=None, export_type='other_text'):
    all_items = list(read_text_format_dir(abs_path, globs=globs, export_type=export_type))
    return pd.DataFrame.from_records(all_items).drop_duplicates('UT')


async def read_text_format_dir_parallel(abs_path, callback, *args, globs=None):
    """ parallel read

    benchmark: read 9232360 item in 1m26s
    """
    loop = asyncio.get_running_loop()
    futures = []
    with concurrent.futures.ProcessPoolExecutor() as pool:
        for path in scan_text_format_dir(abs_path, globs=globs):
            future = loop.run_in_executor(pool, callback, path, *args)
            futures.append(future)
    return await asyncio.gather(*futures, return_exceptions=False)


def _get_uts_parallel_worker(path):
    items = read_text_format_path(path)
    return [item['UT'] for item in items]


async def get_uts_parallel(abs_path):
    """ get all uts
    """
    results = await read_text_format_dir_parallel(abs_path, _get_uts_parallel_worker)
    uts = set()
    for result in results:
        uts.update(result)
    return uts



# coding: utf-8

""" dataset transform
"""

from __future__ import unicode_literals, absolute_import, print_function, division

from optparse import OptionParser
from scilib.db.trans import pgcsv_to_items, pgcsv_to_es, ndjsongz_to_es


def run():
    parser = OptionParser()
    parser.add_option("--from", action="store", type="str", dest="from_path", default="")
    parser.add_option("--from-type", action="store", type="str", dest="from_type", default="")
    parser.add_option("--to", action="store", type="str", dest="to_path", default="")
    parser.add_option("--to-type", action="store", type="str", dest="to_type", default="")

    parser.add_option("--es-index", action="store", type="str", dest="es_index", default="")
    parser.add_option("--es-pk", action="store", type="str", dest="es_pk", default="")

    options, args = parser.parse_args()
    print('options', options)
    trans_type = f'{options.from_type}_to_{options.to_type}'
    trans_funcs = {
        'pgcsv_to_items': lambda options: pgcsv_to_items(options.from_path),
        'pgcsv_to_es': lambda options: pgcsv_to_es(options.from_path, options.es_index, options.es_pk),
        'ndjsongz_to_es': lambda options: ndjsongz_to_es(options.from_path, options.es_index, options.es_pk)
    }
    print('trans_type:', trans_type)
    if trans_type not in trans_funcs:
        print(f'type {trans_type} not support')
        return
    trans_func = trans_funcs[trans_type]
    trans_func(options)


if __name__ == '__main__':
    run()



# coding: utf-8

import os
import subprocess
from scilib.iterlib import chunks

BASE_DIR = os.path.dirname(__file__)
CLI_PATH = os.path.join(BASE_DIR, 'cli.rb')


def batch_classify(names):
    first_names = [i.split()[0].replace(',', '') for i in names]

    output_results = {}
    for chunk in chunks(first_names, size=500):
        output = subprocess.check_output([
            'ruby',
            CLI_PATH,
            *chunk
        ], cwd=BASE_DIR).decode('utf-8')
        output_results.update(dict([i.strip().split(',')[:2] for i in output.split('\n') if i.strip()]))

    results = []
    for first_name in first_names:
        result = output_results.get(first_name, "unknown")
        if result in ['male', 'mostly_male']:
            results.append('male')
        elif result in ['female', 'mostly_female']:
            results.append('female')
        else:
            results.append('unknown')
    return results


def test():
    results = batch_classify(["tom", "lisa", "bob", "test10086"])
    print(results)


if __name__ == '__main__':
    test()



# coding: utf-8

import os
import json
import requests
from .secret import X_API_KEY

BASE_DIR = os.path.dirname(__file__)
DATA_PATH = os.path.join(BASE_DIR, 'data')


def batch_classify(names):
    names = [i.replace('/', '') for i in names]

    output_results = {}
    for name in names:
        file_path = os.path.join(DATA_PATH, f'{name}.json')
        if os.path.exists(file_path):
            with open(file_path) as f:
                data = json.load(f)
                output_results[name] = data['likelyGender']
                continue
        response = requests.get(f'https://v2.namsor.com/NamSorAPIv2/api2/json/genderFull/{name}', headers={
            'X-API-KEY': X_API_KEY
        })
        response.raise_for_status()
        data = response.json()
        print('data', data)
        if 'likelyGender' in data:
            output_results[name] = data['likelyGender']
            with open(file_path, 'w') as f:
                json.dump(data, f)
        else:
            raise ValueError([name, data])

    results = []
    for name in names:
        result = output_results.get(name, "unknown")
        if result in ['male', 'mostly_male']:
            results.append('male')
        elif result in ['female', 'mostly_female']:
            results.append('female')
        else:
            results.append('unknown')
    return results


def test():
    results = batch_classify(["tom", "lisa", "bob", "test10086"])
    print(results)


if __name__ == '__main__':
    test()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import pandas as pd

BASE_DIR = os.path.dirname(__file__)
DATASETS = {
    'wiki': os.path.join(BASE_DIR, 'data/wiki_gender_test.csv'),
    'usa': os.path.join(BASE_DIR, 'data/usa.csv'),
}


def load_test_data(dataset='wiki'):
    items = [dict(item) for index, item in pd.read_csv(DATASETS[dataset]).iterrows()]
    return list(({item['name']: item for item in items}).values())



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

from unittest import TestCase
from scilib.iterlib import uniqify, chunks, isplit


class IterlibTest(TestCase):

    def test_uniqify(self):
        self.assertListEqual(list(uniqify([1, 2, 3])), [1, 2, 3])
        self.assertListEqual(list(uniqify([1, 2, 3, 3])), [1, 2, 3])
        self.assertListEqual(
            list(uniqify([{"id": 1}, {"id": 2}, {"id": 2}], key=lambda x: x["id"])),
            [{"id": 1}, {"id": 2}]
        )

    def test_chunks(self):
        self.assertListEqual(
            list(chunks(range(10), size=1)),
            [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9]]
        )
        self.assertListEqual(
            list(chunks(range(10), size=3)),
            [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
        )
        self.assertListEqual(
            list(chunks(range(10), size=10)),
            [list(range(10))]
        )
        self.assertListEqual(
            list(chunks(range(10), size=100)),
            [list(range(10))]
        )

    def test_isplit(self):
        self.assertListEqual(
            list(isplit(range(10), count=1)),
            [list(range(10))]
        )
        self.assertListEqual(
            list(isplit(range(10), count=4)),
            [[0, 4, 8], [1, 5, 9], [2, 6], [3, 7]]
        )
        self.assertListEqual(
            list(isplit(range(10), count=10)),
            [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9]]
        )
        self.assertListEqual(
            list(isplit(range(10), count=20)),
            [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9], [], [], [], [], [], [], [], [], [], []]
        )



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division


def parse_cr_dois(row):
    tokens = str(row.get('CR', '')).split()
    items = [i.strip(';') for i in tokens if i.startswith('10.')]
    return ';'.join(items)



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import re
import pandas as pd
from optparse import OptionParser

from scilib.gender.gender_guesser_pypi.public import batch_classify


def run(from_excel, to_excel):
    df = pd.read_excel(from_excel)
    items = []
    for index, row in df.iterrows():
        names = re.split(r'[ ,|]', str(row['Authors']))
        if not names:
            items.append(row)
            continue
        name = names[0]
        items.append({
            **row,
            'gender_guesser_name': name,
            'gender_guesser': batch_classify([name + ' '])[0]
        })
    pd.DataFrame.from_records(items).to_excel(to_excel)


if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option("--from-excel", action="store", type="str", dest="from_excel", default=None)
    parser.add_option("--to-excel", action="store", type="str", dest="to_excel", default=None)
    options, args = parser.parse_args()
    run(options.from_excel, options.to_excel)



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os

import random
import pandas as pd
from nltk import NaiveBayesClassifier, classify

BASE_DIR = os.path.dirname(__file__)
DATA_DIR = os.path.join(BASE_DIR, 'data')


def get_name_features(double_names, name, gender=None):
    if '·' in name:
        return
    name = ''.join([i for i in name if i])
    if len(name) not in [2, 3, 4]:
        return
    if name[:2] in double_names or len(name) == 4:
        # first_name = name[:2]
        last_name = name[2:]
    else:
        # first_name = name[:1]
        last_name = name[1:]
    features = {
        'last_name': last_name,
        'last_name_0': None,
        'last_name_1': None,
        'last_name_repeat': len(last_name) == 2 and last_name[0] == last_name[1]
    }

    for index, token in enumerate(last_name):
        features[f'last_name_{index}'] = token
    return features


def load_featureset():
    df_names = pd.read_excel(os.path.join(DATA_DIR, 'names.xlsx'))
    df_double = pd.read_excel(os.path.join(DATA_DIR, 'double_names.xlsx'))
    double_names = set([row['name'] for i, row in df_double.iterrows()])
    print(df_names.shape)

    featureset = []
    for i, row in df_names.iterrows():
        name = str(row['name']).strip()
        gender = str(row['gender']).strip()
        if gender not in ['男', '女']:
            continue
        gender = 'M' if gender == '男' else 'F'
        features = get_name_features(double_names, name, gender=gender)
        if not features:
            continue
        featureset.append((features, gender))

    print(f'load featureset count={len(featureset)}')
    random.shuffle(featureset)
    featureset_m = [i for i in featureset if i[1] == 'M']
    featureset_f = [i for i in featureset if i[1] == 'F']
    print(f'M={len(featureset_m)} F={len(featureset_f)}')
    min_size = min(len(featureset_m), len(featureset_f))
    return (featureset_m[:min_size] + featureset_f[:min_size]), double_names


class ChineseGenderPredictor(object):

    def __init__(self, featureset, double_names):
        self.featureset = featureset
        self.double_names = double_names
        self.training_percent = 0.8
        self.classifier = None
        self.tokens_map = None

    def prepare(self, train_set):
        tokens = []
        tokens_map = {}
        for features, gender in train_set:
            if features['last_name_0']:
                tokens.append((features['last_name_0'], gender))
            if features['last_name_1']:
                tokens.append((features['last_name_1'], gender))
        for token, gender in tokens:
            tokens_map.setdefault(token, {'M': 0, 'F': 0})[gender] += 1
        for k, config in tokens_map.items():
            config['m_percent'] = config['M'] / (config['M'] + config['F'])
            config['f_percent'] = config['F'] / (config['M'] + config['F'])
        self.tokens_map = tokens_map

        for features, gender in self.featureset:
            self.set_full_features(features)

    def set_full_features(self, features):
        pass

    def get_name_full_features(self, name):
        features = get_name_features(self.double_names, name)
        self.set_full_features(features)
        return features

    def train_and_test(self):
        random.shuffle(self.featureset)
        count = len(self.featureset)
        break_point = int(count * self.training_percent)
        train_set = self.featureset[:break_point]
        test_set = self.featureset[break_point:]

        self.prepare(train_set)

        self.classifier = NaiveBayesClassifier.train(train_set)
        accuracy = classify.accuracy(self.classifier, test_set)

        return accuracy

    def get_most_informative_features(self, n=5):
        return self.classifier.most_informative_features(n)

    def predict(self, name):
        features = self.get_name_full_features(name)
        if not features:
            return None, None
        return self.classifier.prob_classify(features).prob('M'), self.classifier.prob_classify(features).prob('F')


def main():
    featureset, double_names = load_featureset()
    predictor = ChineseGenderPredictor(featureset, double_names)
    accuracy = predictor.train_and_test()
    print(accuracy)


if __name__ == '__main__':
    main()



# coding: utf-8

from genderizer.genderizer import Genderizer


def batch_classify(names):
    first_names = [i.split()[0].lower() for i in names]
    results = []
    for first_name in first_names:
        result = Genderizer.detect(firstName=first_name)
        if result in ['male']:
            results.append('male')
        elif result in ['female']:
            results.append('female')
        else:
            results.append('unknown')
    return results



# coding: utf-8

from agefromname import AgeFromName


def batch_classify(names):
    first_names = [i.split()[0].lower() for i in names]

    age_from_name = AgeFromName()
    df_all_result = age_from_name.get_all_name_male_prob()
    result_map = {}
    for index, row in df_all_result.iterrows():
        if row['prob'] > 0.5:
            result_map[str(index)] = 'male'
        elif row['prob'] > 0 and row['prob'] < 0.5:
            result_map[str(index)] = 'female'
        else:
            result_map[str(index)] = 'unknown'

    results = []
    for first_name in first_names:
        if first_name in result_map:
            results.append(result_map[first_name])
        else:
            results.append('unknown')
    return results



# coding: utf-8

""" import WOS data
"""

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import hashlib
import asyncio
import pandas as pd
from optparse import OptionParser
from scilib.iterlib import chunks

from scilib.wos.importer import read_text_format_dir_parallel, read_text_format_path
from scilib.wos.parser import parse_version1, remove_row_type
from scilib.db.es import index_or_update_rows


def es_callback(path, index):
    items = read_text_format_path(path)
    parse_version1(items)
    index_or_update_rows(items, index=index, action='index')


def csv_callback(path, fields, tag, py_from, py_to):
    fields = [i for i in fields.split(',') if i]

    items = read_text_format_path(path)
    parse_version1(items)
    if tag:
        tags = set([i.strip() for i in tag.split(',') if i.strip()])
        items = [item for item in items if set(item['tags']).issuperset(tags)]
    if py_from and py_to:
        pys = [str(i) for i in range(int(py_from), int(py_to) + 1)]
        items = [item for item in items if item.get('PY', '') in pys]

    for item in items:
        remove_row_type(item)
    return [{k: v for k, v in item.items() if not fields or k in fields} for item in items]


def fast5k_csv_callback(path):
    return read_text_format_path(path, export_type='fast5k_csv')


def count_callback(path):
    items = read_text_format_path(path)
    return len(items)


def cr_query_callback(path, tag):
    items = read_text_format_path(path)
    parse_version1(items)
    download_dois = set([item['DI'] for item in items if str(item.get('DI', '')).startswith('10.')])
    if tag:
        tags = set([i.strip() for i in tag.split(',') if i.strip()])
        items = [item for item in items if set(item['tags']).issuperset(tags)]
    cr_dois = set()
    for item in items:
        cr_dois.update(item['cr_dois'])
    return download_dois, cr_dois


def ut_query_callback(path, to_dir):
    items = read_text_format_path(path)
    download_uts = set([item['UT'] for item in items if item.get('UT', '')])
    for uts in chunks(download_uts, size=500):
        if not uts:
            continue
        text = f'UT=({" or ".join(uts)})'
        file_path = os.path.join(to_dir, hashlib.md5(text.encode('utf-8')).hexdigest() + '.txt')
        with open(file_path, 'w') as f:
            f.write(text)
            print(len(uts), file_path)


async def main(from_dir, to_type, to_dir, index, fields, export_type, tag, py_from, py_to):
    if to_type == 'es':
        await read_text_format_dir_parallel(from_dir, es_callback, index)
    elif to_type == 'count':
        results = await read_text_format_dir_parallel(from_dir, count_callback)
        print(sum(results))
    elif to_type == 'cr_query':
        results = await read_text_format_dir_parallel(from_dir, cr_query_callback, tag)
        download_dois = set.union(*[set(li[0]) for li in results])
        cr_dois = set.union(*[set(li[1]) for li in results])
        new_dois = cr_dois - download_dois
        print(f'download_dois={len(download_dois)} cr_dois={len(cr_dois)} new_dois={len(new_dois)}')
    elif to_type == 'ut_query':
        await read_text_format_dir_parallel(from_dir, ut_query_callback, to_dir)
    elif to_type.endswith('.fast5k.csv'):
        results = await read_text_format_dir_parallel(from_dir, fast5k_csv_callback)
        df = pd.DataFrame.from_records((i for li in results for i in li)).drop_duplicates('UT')
        print(df.shape)
        df.to_csv(to_type)
    elif to_type.endswith('.csv'):
        results = await read_text_format_dir_parallel(from_dir, csv_callback, fields, tag, py_from, py_to)
        df = pd.DataFrame.from_records((i for li in results for i in li)).drop_duplicates('UT')
        print(df.shape)
        df.to_csv(to_type)
    elif to_type.endswith('.xlsx'):
        results = await read_text_format_dir_parallel(from_dir, csv_callback, fields, tag, py_from, py_to)
        df = pd.DataFrame.from_records((i for li in results for i in li)).drop_duplicates('UT')
        print(df.shape)
        df.to_excel(to_type)
    else:
        raise ValueError(to_type)


def run():
    parser = OptionParser()
    parser.add_option("--from", action="store", type="str", dest="from_dir", default=".")
    parser.add_option("--to", action="store", type="str", dest="to", default="count")
    parser.add_option("--to-dir", action="store", type="str", dest="to_dir", default="/tmp")
    parser.add_option("--index", action="store", type="str", dest="index", default="wos")
    parser.add_option("--fields", action="store", type="str", dest="fields", default="")
    parser.add_option("--export_type", action="store", type="str", dest="export_type", default="other_text")

    parser.add_option("--tag", action="store", type="str", dest="tag", default=None)
    parser.add_option("--py_from", action="store", type="str", dest="py_from", default=None)
    parser.add_option("--py_to", action="store", type="str", dest="py_to", default=None)

    options, args = parser.parse_args()
    asyncio.run(main(
        options.from_dir,
        options.to,
        options.to_dir,
        options.index,
        options.fields,
        options.export_type,
        options.tag,
        options.py_from,
        options.py_to,
    ))


if __name__ == '__main__':
    run()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import asyncio
from unittest import TestCase

from scilib.wos.importer import read_text_format_dir, read_text_format_dir_as_pd, get_uts_parallel

TEST_PATH = os.path.dirname(__file__)


class WOSImporterTest(TestCase):

    def test_read_text_format_dir(self):
        items = list(read_text_format_dir(TEST_PATH))

        self.assertEqual(len(items), 500)
        for item in items:
            for key in item.keys():
                self.assertEqual(len(key), 2)
            self.assertTrue(item['UT'].startswith('WOS:'))

    def test_read_text_format_dir_as_pd(self):
        df = read_text_format_dir_as_pd(TEST_PATH)
        self.assertEqual(df.shape[0], 500)

    def test_get_uts_parallel(self):
        uts = asyncio.run(get_uts_parallel(TEST_PATH))
        self.assertEqual(len(uts), 500)



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

from .parse_country import parse_country

ORG_MAP = {
    "Peking Univ": "Peking Univ",
    "Beijing Univ": "Peking Univ",
    "Tsinghua Univ": "Tsinghua Univ",
    "Tsing Hua Univ": "Tsinghua Univ",
    "Xi An Jiao Tong Univ": "Xi An Jiao Tong Univ",
    "Xian Jiaotong Univ": "Xi An Jiao Tong Univ",
    "Renmin Univ China": "Renmin Univ China",
    "Renmin Univ": "Renmin Univ China",
    "E China Normal Univ": "E China Normal Univ",
    "East China Normal Univ": "E China Normal Univ",
    "Southwestern Univ Finance & Econ": "Southwestern Univ Finance & Econ",
    "SW Univ Finance & Econ": "Southwestern Univ Finance & Econ",
    "S China Univ Technol": "S China Univ Technol",
    "South China Univ Technol": "S China Univ Technol",
    "South China Normal Univ": "South China Normal Univ",
    "South China Normal Univ": "South China Normal Univ",
}


def parse_address(text):
    """ parse name and address from WOS C1 field
    """
    if not text or str(text) == 'nan':
        return []
    state = 'NAME'  # NAME | ADDRESS | ADDRESS_END
    name = ''
    address = ''

    results = []
    for c in text:
        if state == 'NAME':
            if c == ']':
                state = 'ADDRESS'
                continue
            elif c == '[':
                continue
            else:
                name += c
                continue
        elif state == 'ADDRESS':
            if c == '[':
                results.append((name, address))
                state = 'NAME'
                name = ''
                address = ''
                continue
            elif c == ' ' and address == '':
                continue
            else:
                address += c
                continue
        else:
            raise ValueError(state)

    if name and address:
        results.append((name, address))
    return results


def parse_org(text):
    tokens = [i.strip() for i in text.split(',') if i.strip()]
    if tokens and tokens[0]:
        return ORG_MAP.get(tokens[0], tokens[0])


def parse_address_info(text, *, hmt=True):
    results = parse_address(text)
    infos = []
    for name, address in results:
        org = parse_org(address)
        country = parse_country(dict(C1=address), field='C1', hmt=hmt)
        infos.append(dict(
            name=name,
            address=address,
            org=org,
            country=country,
        ))

    return infos



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

from .parse_categorys import SSH_CATEGORYS_SET


def parse_tags(row):
    """ tags

    require:
        - is_article
        - countrys_c1_rp
        - ecoom_categorys
    """

    tags = []
    country_tag_map = {
        'china': 'mainland_article',
        'hong kong': 'hk_article',
        'macao': 'macao_article',
        'taiwan': 'taiwan_article',
        'usa': 'usa_article',
        'japan': 'japan_article',
        'uk': 'uk_article',
    }

    for key, value in country_tag_map.items():
        if row['is_article'] and key in row['countrys_c1_rp']:
            tags.append(value)

    ecoom_categorys = [i.strip().lower() for i in str(row['ecoom_categorys']).split(';')]
    if SSH_CATEGORYS_SET & set(ecoom_categorys):
        tags.append('ecoom7')

    return ';'.join(tags)



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import json
import subprocess
from pathlib import Path

from .base import call_batch
from .plugin import start_with_cd, xls2dta, summary, reg, nbreg

STATA_ENTRY = os.environ.get('STATA_ENTRY', '/Applications/Stata/StataSE.app/Contents/MacOS/StataSE')


def run(working_dir):
    """ https://www.stata.com/support/faqs/mac/advanced-topics/
    """
    with open(os.path.join(working_dir, 'config.json')) as f:
        config = json.load(f)
    actions = [
        start_with_cd(working_dir),
        xls2dta('use-data.xlsx', 'use-data.dta'),
    ]
    for action in config['actions']:
        if action['type'] == 'summary':
            actions.append(summary(action['vars']))
        elif action['type'] == 'nbreg':
            actions.append(reg(action["vars"]))
            actions.append(nbreg(f'{action["depVar"]} {action["vars"]}'))

    do_content = call_batch(*actions)
    do_file = os.path.join(working_dir, 'run.do')
    with open(do_file, 'w') as f:
        f.write(do_content)

    try:
        subprocess.call([
            STATA_ENTRY,
            '-b',
            '-e',
            'do',
            'run.do'
        ], cwd=working_dir, timeout=3 * 60)
    except subprocess.TimeoutExpired:
        print('subprocess.TimeoutExpired')


def run_all(entry_dir):
    # for dir_name in os.listdir(entry_dir):
    for file in Path(entry_dir).glob('**/config.json'):
        # working_dir = os.path.join(entry_dir, dir_name)
        working_dir = os.path.dirname(file)
        if not (os.path.isdir(working_dir)):
            continue
        print(f'run with {working_dir}...')
        run_log = os.path.join(working_dir, 'run.log')
        if os.path.exists(run_log):
            print(f'ignore {working_dir}')
            continue
        run(working_dir)


if __name__ == '__main__':
    run_all(os.environ['DIR_AUTOPROCESS'])



# coding: utf-8

import os
import urllib
from zipfile import ZipFile
import csv
import pickle

BASE_DIR = os.path.dirname(__file__)
FILE_PICKLE = os.path.join(BASE_DIR, "data/names.pickle")
FILE_ZIP = os.path.join(BASE_DIR, "data/names.zip")


def getNameList():
    if not os.path.exists(FILE_PICKLE):
        # print("names.pickle does not exist, generating")
        if not os.path.exists(FILE_ZIP):
            print("names.zip does not exist, downloading from github")
            downloadNames()
        else:
            print("names.zip exists, not downloading")

        # print("Extracting names from names.zip")
        namesDict = extractNamesDict()
        maleNames = list()
        femaleNames = list()

        # print("Sorting Names")
        for name in namesDict:
            counts = namesDict[name]
            tuple = (name, counts[0], counts[1])
            if counts[0] > counts[1]:
                maleNames.append(tuple)
            elif counts[1] > counts[0]:
                femaleNames.append(tuple)

        names = (maleNames, femaleNames)

        print("Saving names.pickle")
        fw = open(FILE_PICKLE, "wb")
        pickle.dump(names, fw, -1)
        fw.close()
        print("Saved names.pickle")
    else:
        print("names.pickle exists, loading data")
        f = open(FILE_PICKLE, "rb")
        names = pickle.load(f)
        print("names.pickle loaded")

    print(
        "%d male names loaded, %d female names loaded" % (len(names[0]), len(names[1]))
    )

    return names


def downloadNames():
    os.system("wget https://github.com/downloads/sholiday/genderPredictor/names.zip")
    return
    u = urllib.urlopen(
        "https://github.com/downloads/sholiday/genderPredictor/names.zip"
    )
    localFile = open(FILE_ZIP, "w")
    localFile.write(u.read())
    localFile.close()


def extractNamesDict():
    zf = ZipFile(FILE_ZIP, "r")
    filenames = zf.namelist()

    names = dict()
    genderMap = {"M": 0, "F": 1}

    for filename in filenames:
        file = zf.open(filename, "r")
        lines = (line.decode("ascii") for line in file)
        rows = csv.reader(lines)

        for row in rows:
            name = row[0].upper()
            gender = genderMap[row[1]]
            count = int(row[2])

            if name not in names:
                names[name] = [0, 0]
            names[name][gender] = names[name][gender] + count

        file.close()
        print("\tImported %s" % filename)
    return names


if __name__ == "__main__":
    getNameList()




# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

from aiohttp import web
from scilib.gender.benchmark.benchmark import CONFIGS_MAP
import redis

KEY_PREFIX = 'scilib-gender'
POOL = redis.ConnectionPool()


async def gender(request):
    tool = request.query.get('tool', 'gender_guesser')
    names = request.query.get('name', None)
    if tool not in CONFIGS_MAP:
        return web.json_response(dict(status='error', message_type='invalid_tool'))
    if not names:
        return web.json_response(dict(status='error', message_type='invalid_name'))

    func = CONFIGS_MAP[tool][1]
    names = [i.strip() for i in names.split(';') if i.strip()]

    conn = redis.Redis(connection_pool=POOL)
    results = conn.hmget(f'{KEY_PREFIX}-{tool}', keys=names)
    results = [i.decode('utf-8') if i else None for i in results]

    results_items = [[i, r] for i, r in enumerate(results)]
    blank_items = [[i, r] for i, r in results_items if not r]
    if blank_items:
        call_results = func([names[i] for i, r in blank_items])
        for i, r in enumerate(call_results):
            results_items[blank_items[i][0]][1] = r

    results = [r for i, r in results_items]
    if not all(results):
        return web.json_response(dict(status='error', message_type='invalid_results', results_items=results_items))

    if blank_items:
        conn.hset(f'{KEY_PREFIX}-{tool}', mapping={names[i]: r for i, r in results_items})

    return web.json_response(dict(
        status='success',
        results=[dict(name=names[i], result=r) for i, r in results_items],
    ))


def main():
    app = web.Application()
    app.add_routes([
        web.get('/gender', gender)
    ])
    web.run_app(app, port=9901)


if __name__ == '__main__':
    main()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division
import re
from libs.iterlib import uniqify


def parse_is_article(row):
    if 'article' in str(row['DT']).lower():
        return True
    return False


def parse_document_types(row):
    dt = str(row.get('DT', ''))
    return ';'.join([i.strip() for i in dt.split(';')])


def parse_is_oa(row):
    oa = str(row.get('OA', '')).lower()
    if oa and oa != 'nan':
        return True
    return False


def parse_py_datetime(row):
    if row.get('PY', '') and str(row['PY']) != 'nan':
        return f'{row["PY"]}-01-01'
    return None


def parse_fu_tokens(row):
    if row.get('FU', '') and str(row['FU']) != 'nan':
        tokens = [re.sub(r'[^a-zA-Z0-9]', '', i) for i in re.split(r'[^a-zA-Z0-9]', row['FU'])]
        return ';'.join(uniqify([i for i in tokens if i]))
    return ''



# coding: utf-8

""" import cnki data
"""

from __future__ import unicode_literals, absolute_import, print_function, division

import asyncio
import pandas as pd
from optparse import OptionParser
from scilib.cnki.importer import read_text_format_dir, read_spider_format_dir, collect_keywords
from scilib.cnki.importer import read_spider_format, read_spider_format_dir_parallel
from scilib.db.es import index_or_update_rows


def es_callback(path):
    try:
        items = list(read_spider_format(path))
        print(len(items))
        index_or_update_rows(items, index='cnki_nsfc', action='index', pk='list_id')
    except Exception as e:
        print(e)
        return


async def main(from_dir, to, from_type, to_type):
    if from_type == 'text':
        items = read_text_format_dir(from_dir)
        df = pd.DataFrame.from_records(items)
        df = df.drop_duplicates(subset=['Title'])
    elif from_type == 'spider':
        items = read_spider_format_dir(from_dir)
        df = pd.DataFrame.from_records(items)
        df = df.drop_duplicates(subset=['Title'])
    elif from_type == 'spider_parallel':
        if to_type == 'parallel_es':
            await read_spider_format_dir_parallel(from_dir, callback=es_callback)
    else:
        raise ValueError(f'unknown from type: {from_type}')

    if to_type == 'corr':
        print('df.shape', df.shape)
        df.to_excel(to)
        counter, counter_map, years_items_flat, corrs = collect_keywords([dict(i) for index, i in df.iterrows()])
        counter_items = [dict(k=k, v=v) for k, v in counter.most_common()]
        pd.DataFrame.from_records(counter_items).to_excel(to + '.counter.xlsx')
        pd.DataFrame.from_records(years_items_flat).to_excel(to + '.years_items_flat.xlsx')
        pd.DataFrame.from_records(corrs).to_excel(to + '.corrs.xlsx')
        pd.DataFrame.from_records(corrs).to_csv(to + '.corrs.csv')
    elif to_type == 'excel':
        df.to_excel(to)
    elif to_type == 'parallel_es':
        pass
    else:
        raise ValueError(f'unknown to type: {to_type}')


def run():
    parser = OptionParser()
    parser.add_option("--from", action="store", type="str", dest="from_dir", default=".")
    parser.add_option("--to", action="store", type="str", dest="to", default="count")
    parser.add_option("--from-type", action="store", type="str", dest="from_type", default="text")
    parser.add_option("--to-type", action="store", type="str", dest="to_type", default="corr")

    options, args = parser.parse_args()
    asyncio.run(main(options.from_dir, options.to, options.from_type, options.to_type))


if __name__ == '__main__':
    run()



# coding: utf-8

import random
from nltk import NaiveBayesClassifier, classify
from .USSSALoader import getNameList


class genderPredictor:

    def getFeatures(self):
        maleNames, femaleNames = self._loadNames()

        featureset = list()

        for nameTuple in maleNames:
            features = self._nameFeatures(nameTuple[0])
            male_prob, female_prob = self._getProbDistr(nameTuple)
            features["male_prob"] = male_prob
            features["female_prob"] = female_prob
            featureset.append((features, "M"))

        for nameTuple in femaleNames:
            features = self._nameFeatures(nameTuple[0])
            male_prob, female_prob = self._getProbDistr(nameTuple)
            features["male_prob"] = male_prob
            features["female_prob"] = female_prob
            featureset.append((features, "F"))

        return featureset

    def trainAndTest(self, trainingPercent=0.80):
        featureset = self.getFeatures()
        random.shuffle(featureset)

        name_count = len(featureset)

        cut_point = int(name_count * trainingPercent)

        train_set = featureset[:cut_point]
        test_set = featureset[cut_point:]

        self.train(train_set)

        return self.test(test_set)

    def classify(self, name):
        feats = self._nameFeatures(name)
        return self.classifier.classify(feats)

    def train(self, train_set):
        self.classifier = NaiveBayesClassifier.train(train_set)
        return self.classifier

    def test(self, test_set):
        return classify.accuracy(self.classifier, test_set)

    def _getProbDistr(self, nameTuple):
        male_prob = (nameTuple[1] * 1.0) / (nameTuple[1] + nameTuple[2])
        if male_prob == 1.0:
            male_prob = 0.99
        elif male_prob == 0.0:
            male_prob = 0.01
        else:
            pass
        female_prob = 1.0 - male_prob
        return (male_prob, female_prob)

    def getMostInformativeFeatures(self, n=5):
        return self.classifier.most_informative_features(n)

    def _loadNames(self):
        return getNameList()

    def _nameFeatures(self, name):
        name = name.upper()
        return {
            "last_letter": name[-1],
            "last_two": name[-2:],
            "last_three": name[-3:],
            "last_is_vowel": (name[-1] in "AEIOUY"),
        }


if __name__ == "__main__":
    gp = genderPredictor()
    accuracy = gp.trainAndTest()
    print("Accuracy: %f" % accuracy)
    print("Most Informative Features")
    feats = gp.getMostInformativeFeatures(10)
    for feat in feats:
        print("\t%s = %s" % feat)
    name = input("Enter name to classify: ")
    print("\n%s is classified as %s" % (name, gp.classify(name)))



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import json
import requests
from libs.iterlib import chunks

ES_API = os.environ.get('ES_API', 'http://localhost:9205')
ES_BULK_API = f'{ES_API}/_bulk'


def index_or_update_rows(rows, *, index="wos", action="index", pk="UT"):
    """ index_or_update_rows

    https://www.elastic.co/guide/en/elasticsearch/reference/current/docs-bulk.html
    """
    for chunk in chunks(rows, size=2000):
        _index_or_update_rows(chunk, index=index, action=action, pk=pk)


def _index_or_update_rows(rows, *, index="wos", action="index", pk="UT"):
    if not rows:
        return
    lines = []
    for row in rows:
        lines.append({
            action: {
                "_index": index,
                "_id": row[pk],
            }
        })
        if action == "update":
            lines.append(dict(doc=row))
        elif action == "index":
            lines.append(row)
        else:
            raise ValueError(action)

    contents = [json.dumps(line) for line in lines]
    content = '\n'.join(contents) + '\n'
    response = requests.post(ES_BULK_API, data=content, headers={
        "Content-Type": "application/x-ndjson",
    })
    response_json = response.json()

    if 'errors' not in response_json:
        print(response_json)
        raise ValueError(response_json)
    elif response_json['errors']:
        print(response_json)
        raise ValueError(response_json)
    else:
        print('took', response_json['took'])



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
from unittest import TestCase

from scilib.wos.importer import read_text_format_dir_as_pd
from scilib.wos.parse_country import add_countrys_to_df
from scilib.wos.parse_categorys import SSH_CATEGORYS_SET, parse_ecoom_categorys
from scilib.wos.parse_doi import parse_cr_dois
from scilib.wos.parse_orcid import parse_orcid, parse_orcid_info
from scilib.wos.parse_rid import parse_rid
from scilib.wos.parse_address import parse_address, parse_address_info
from scilib.wos.parser import parse_version1

TEST_PATH = os.path.dirname(__file__)


class WOSParserTest(TestCase):

    def test_add_countrys_to_df(self):
        df = read_text_format_dir_as_pd(TEST_PATH)
        df['countrys_c1'] = add_countrys_to_df(df, field='C1', hmt=True)
        countrys = [row['countrys_c1'] for index, row in df.iterrows()]
        countrys_join = ';'.join(i for i in countrys if i)

        self.assertTrue('china' in countrys_join)
        self.assertTrue('usa' in countrys_join)
        self.assertTrue('hong kong' in countrys_join)

    def test_parse_ecoom_categorys(self):
        df = read_text_format_dir_as_pd(TEST_PATH)
        count = 0
        for index, row in df.iterrows():
            ecoom_categorys = parse_ecoom_categorys(row)
            if [c for c in SSH_CATEGORYS_SET if c in ecoom_categorys]:
                count += 1
        self.assertTrue(count, 356)  # checked

    def test_parse_cr_dois(self):
        df = read_text_format_dir_as_pd(TEST_PATH)
        dois = []
        for index, row in df.iterrows():
            cr_dois = parse_cr_dois(row)
            dois.extend([i for i in cr_dois.split(';') if i])
        self.assertTrue(len(dois), 29451)  # checked

    def test_parse_version1(self):
        df = read_text_format_dir_as_pd(TEST_PATH)
        items = [dict(row) for i, row in df.iterrows()]

        parse_version1(items)
        for item in items:
            self.assertEqual(item['is_article'], 'article' in str(item['DT']).lower())
            self.assertEqual('10.' in ';'.join(item['cr_dois']), '10.' in str(item['CR']))
            self.assertEqual('hong kong' in ';'.join(item['countrys_c1']), 'hong kong' in str(item['C1']).lower())

            # For debug:
            # print('-' * 80)
            # for key in item.keys():
            #     if len(key) > 2:
            #         print(key, item[key])

    def test_parse_address(self):
        df = read_text_format_dir_as_pd(TEST_PATH)
        items = [dict(row) for i, row in df.iterrows()]

        for item in items:
            parse_address(item['C1'])
            parse_address_info(item['C1'])

        text = """
[Liu, Jian] Nanjing Normal Univ, Sch Geog Sci, Minist Educ, Key Lab Virtual Geog Environm, Nanjing 210023, Jiangsu, Peoples R China.;
[Liu, Jian] Chinese Acad Sci, Nanjing Inst Geog & Limnol, State Key Lab Lake Sci & Environm, Nanjing 210008, Jiangsu, Peoples R China.;
[Wang, Bin; Yim, So-Young; Lee, June-Yi] Univ Hawaii Manoa, Int Pacific Res Ctr, Honolulu, HI 96825 USA.;
[Wang, Bin; Yim, So-Young; Lee, June-Yi] Univ Hawaii Manoa, Dept Meteorol, Honolulu, HI 96825 USA.;
[Cane, Mark A.] Columbia Univ, Lamont Doherty Earth Observ, Palisades, NY 10964 USA.
        """.strip().replace('\n', '') # noqa
        results = parse_address(text)

        self.assertEqual(results[0][0], 'Liu, Jian')
        self.assertEqual(results[4][0], 'Cane, Mark A.')

        infos = parse_address_info(text)
        self.assertEqual(infos[0]['org'], 'Nanjing Normal Univ')
        self.assertEqual(infos[4]['org'], 'Columbia Univ')
        self.assertEqual(infos[0]['country'], 'china')
        self.assertEqual(infos[4]['country'], 'usa')

    def test_parse_orcid(self):
        df = read_text_format_dir_as_pd(TEST_PATH)
        items = [dict(row) for i, row in df.iterrows()]

        for item in items:
            parse_orcid(item['OI'])
            parse_orcid_info(item['OI'])

        first_results = parse_orcid_info(items[0]['OI'])
        self.assertEqual(first_results[0]['name'], 'Casula, Manuela')
        self.assertEqual(first_results[0]['orcid'], '0000-0002-5124-5361')

    def test_parse_rid(self):
        text = """
Bechter, Karl/AAH-6637-2020; Tian, Li/P-2950-2015; Otto,; Markus/F-4304-2015; Benros, Michael Eriksen/N-5868-2016; Muller,; Sabine/P-4279-2015
        """.strip().replace('\n', '') # noqa
        results = parse_rid(text)
        self.assertEqual(results[0][0], 'Bechter, Karl')
        self.assertEqual(results[0][1], 'AAH-6637-2020')

        df = read_text_format_dir_as_pd(TEST_PATH)
        items = [dict(row) for i, row in df.iterrows()]

        for item in items:
            parse_rid(item['RI'])



# coding: utf-8

import gender_guesser.detector as gender


def batch_classify(names):
    first_names = [i.split()[0] for i in names]
    results = []
    d = gender.Detector(case_sensitive=False)
    for first_name in first_names:
        result = d.get_gender(first_name)
        if result in ['male', 'mostly_male']:
            results.append('male')
        elif result in ['female', 'mostly_female']:
            results.append('female')
        else:
            results.append('unknown')
    return results


def test():
    d = gender.Detector(case_sensitive=False)
    print(d.get_gender(u"Bob"))


if __name__ == '__main__':
    test()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import numpy as np
import pandas as pd
from scipy.io import loadmat

BASE_DIR = os.path.dirname(__file__)
WIKI_MAT_PATH = os.path.join(BASE_DIR, 'data/wiki.mat')
WIKI_CSV_PATH = os.path.join(BASE_DIR, 'data/wiki.csv')
WIKI_GENDER_TEST_CSV_PATH = os.path.join(BASE_DIR, 'data/wiki_gender_test.csv')


def mat_data_to_csv():
    mat_data = loadmat(WIKI_MAT_PATH)
    fields = [
        'dob',
        'photo_taken',
        'full_path',
        'gender',
        'name',
        'face_location',
        'face_score',
        'second_face_score',
    ]

    wiki_data = mat_data['wiki']
    array_data = [
        wiki_data[0][0][0][0],
        wiki_data[0][0][1][0],
        [i[0] for i in wiki_data[0][0][2][0]],
        wiki_data[0][0][3][0],
        [i[0] if i.size > 0 else '' for i in wiki_data[0][0][4][0]],
        wiki_data[0][0][5][0],
        wiki_data[0][0][6][0],
        wiki_data[0][0][7][0],
    ]

    np_data = np.vstack(array_data).T
    df = pd.DataFrame(np_data)
    df.columns = fields
    df.to_csv(WIKI_CSV_PATH, index=False)

    # gender test
    items = []
    for index, row in df.iterrows():
        if row['name'] and row['gender'] in [0, 1]:
            items.append(dict(name=row['name'], gender='male' if row['gender'] == 1 else 'female'))
    pd.DataFrame.from_records(items).to_csv(WIKI_GENDER_TEST_CSV_PATH, index=False)


if __name__ == '__main__':
    mat_data_to_csv()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import re
import pandas as pd
from scilib.gender.imdb_wiki_dataset.public import load_test_data
from scilib.gender.gender_predictor.public import batch_classify as batch_classify1
from scilib.gender.gender_guesser_pypi.public import batch_classify as batch_classify2
from scilib.gender.genderizer_pypi.public import batch_classify as batch_classify3
from scilib.gender.agefromname_pypi.public import batch_classify as batch_classify4
from scilib.gender.gender_r.public import batch_classify as batch_classify5
from scilib.gender.go_gender.public import batch_classify as batch_classify6
from scilib.gender.gender_detector.public import batch_classify as batch_classify7
from scilib.gender.gender_computer.public import batch_classify as batch_classify8

from scilib.gender.api_genderize.public import batch_classify as batch_classify11
from scilib.gender.api_namsor.public import batch_classify as batch_classify12
from scilib.gender.api_gender.public import batch_classify as batch_classify13

BASE_DIR = os.path.dirname(__file__)
CSV_PATH = os.path.join(BASE_DIR, 'results.csv')
MD_PATH = os.path.join(BASE_DIR, 'README.md')
CONFIGS = [
    ['gender-api.com', batch_classify13, dict(limit=1000)],
    ['genderize.io', batch_classify11, dict(limit=2000)],
    ['namsor.com', batch_classify12, dict(limit=5000)],

    ['gender_predictor', batch_classify1, {}],
    ['gender_guesser', batch_classify2, {}],
    ['genderizer', batch_classify3, {}],
    ['gender@R', batch_classify5, {}],
    ['gender@R-USA', batch_classify5, dict(dataset='usa')],
    ['agefromname', batch_classify4, {}],
    ['gender@go', batch_classify6, {}],
    ['gender_detector', batch_classify7, {}],
    ['genderComputer', batch_classify8, {}],
]
CONFIGS_MAP = {i[0]: i for i in CONFIGS}


def _get_result_summary(test_data, results):
    metrics = {
        'total_count': len(results),
        'male-male': 0,
        'male-female': 0,
        'male-unknown': 0,
        'female-male': 0,
        'female-female': 0,
        'female-unknown': 0,
    }
    for index, item in enumerate(test_data):
        key = item['gender'] + '-' + results[index]
        if key not in metrics:
            raise ValueError(key)
        metrics[key] += 1

    m = metrics
    m['accuracy1'] = (m['male-male'] + m['female-female']) / m['total_count']
    m['accuracy2'] = (m['male-male'] + m['female-female']) / (m['male-female'] + m['female-male'] + m['male-male'] + m['female-female'])  # noqa
    m['score1'] = (m['male-female'] + m['male-unknown'] + m['female-male'] + m['female-unknown']) / m['total_count']
    m['score2'] = (m['male-female'] + m['female-male']) / (m['male-female'] + m['female-male'] + m['male-male'] + m['female-female'])  # noqa
    m['score3'] = (m['male-unknown'] + m['female-unknown']) / m['total_count']
    m['score4'] = (m['male-female'] - m['female-male']) / (m['male-female'] + m['female-male'] + m['male-male'] + m['female-female']) # noqa
    return m


def run():
    benchmark_results = []
    for name, method, options in CONFIGS:
        print(f'\n\nbenckmark: {name} start')
        dataset = options.get('dataset') or 'wiki'
        test_data = load_test_data(dataset)
        names = [item['name'] for item in test_data]
        limit = options.get('limit') or len(test_data)
        results = method(names[:limit])
        metrics = _get_result_summary(test_data[:limit], results)
        print(metrics)
        benchmark_results.append(dict(name=name, **metrics))

    df = pd.DataFrame.from_records(benchmark_results)
    df.to_csv(CSV_PATH, index=False)

    markdown_content = df.to_markdown(index=False)
    with open(MD_PATH, 'r+') as f:
        content = f.read()
        new_content = re.sub(r'(?sm)(BENCHMARK_START -->).*(<!-- BENCHMARK_END)', rf'\1\n{markdown_content}\n\2', content)  # noqa
        f.seek(0)
        f.write(new_content)


if __name__ == '__main__':
    run()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os

BASE_DIR = os.path.abspath(os.path.dirname(__file__))


def call(*args):
    return ' '.join(args)


def call_batch(*line):
    return '\n'.join(line)


def call_quietly(code):
    return call_batch(
        call('qui{'),
        code,
        call('}'),
    )


def call_from_file(do_file, **kwargs):
    with open(os.path.join(BASE_DIR, do_file)) as f:
        content = f.read()
    for k, v in kwargs.items():
        content = content.replace(k, v)
    return content



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import re
from functools import lru_cache, partial
import pandas as pd

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
CHMT = set(['hong kong', 'macao', 'taiwan', 'china'])
COUNTRY_MAP = {
    "England": "UK",
    "Scotland": "UK",
    "Wales": "UK",
}
HMT_MAP = {
    'hong kong': ['hong kong', 'hongkong'],
    'macao': ['macao', 'macau'],
}


@lru_cache(maxsize=None)
def _get_std_country_list():
    df_std_country = pd.read_excel(os.path.join(BASE_DIR, 'configs/std_country_map.xlsx'))
    return [
        {'end': row['end'].lower().strip().replace('#', ' '), 'country': row['country'].lower().strip()}
        for i, row in df_std_country.iterrows()
    ]


def parse_country(
    row,
    *,
    field='C1',
    extra_field=None,
    hmt=False,
):
    std_country_list = _get_std_country_list()
    address_list = str(row.get(field, ''))
    if extra_field:
        address_list += ';'
        address_list += str(row.get(extra_field, ''))
    address_list = re.sub(r'\[.*?\]', '', address_list)
    address_list = address_list.split(';')
    countrys = []
    nomatchs = []

    for address in address_list:
        address = address.replace('.', '').lower()
        if address == 'nan':
            continue

        # hmt
        try:
            for key, tokens in HMT_MAP.items():
                for token in tokens:
                    if token in address:
                        if hmt:
                            countrys.append(key)
                        else:
                            countrys.append('china')
                        raise StopIteration('match')
        except StopIteration:
            continue

        country_list = re.split(' |,', address)
        country_list_join = ' '.join(country_list)
        for row in std_country_list:
            if country_list_join.endswith(row['end']):
                parsered_country = row['country']
                countrys.append(parsered_country)
                break
        else:
            if re.search(r'[a-z]{2} \d{5}$', address):
                countrys.append('usa')
            elif re.search('(xiamen|china)', address):
                countrys.append('china')
            elif re.match(r'^[a-z]+, *[a-z]+$', address.strip()):
                continue
            else:
                nomatchs.append(address)

    return ';'.join(countrys)


def add_countrys_to_df(
    df_temp,
    *,
    field='C1',
    extra_field=None,
    hmt=False,
):
    _apply_col = partial(parse_country, field=field, extra_field=extra_field, hmt=hmt)
    return df_temp.apply(_apply_col, axis=1)


def parse_collaboration_type_china(countrys):
    countrys = set(countrys.split(';'))
    if len(countrys) == 1:
        return 'N-ICP'
    elif countrys.issubset(CHMT):
        return 'chmt'
    elif countrys:
        return 'ICP'
    else:
        return 'NODATA'


def parse_lead_type_china(countrys):
    countrys_list = countrys.split(';')
    if countrys_list:
        if countrys_list[0] in CHMT:
            return countrys_list[0]
        return 'other'
    return 'NODATA'


def parse_collaboration_type(countrys):
    countrys = set(countrys.split(';'))
    if len(countrys) == 1:
        return 'N-ICP'
    elif countrys:
        return 'ICP'
    else:
        return 'NODATA'


def parse_first_country(countrys):
    countrys_list = countrys.split(';')
    if countrys_list:
        return countrys_list[0]
    return 'NODATA'



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
from unittest import TestCase

from scilib.iolib.file import create_temp_dir, remove_temp_dir, write_file, read_file


class IolibFileTest(TestCase):

    def test_temp(self):
        temp_dir = create_temp_dir()
        try:
            file_path = os.path.join(temp_dir, 'test.txt')
            write_file(file_path, temp_dir)
            content = read_file(file_path)
            self.assertEqual(content, temp_dir)
        finally:
            remove_temp_dir(temp_dir)



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
from functools import lru_cache
import pandas as pd

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
SSH_CATEGORYS_SET = set([
    'education, media & information science',
    'sociology & anthropology',
    'community & social issues',
    'business, economics, planning',
    'political science & administration',
    'law',
    'special psychiatry',
])


@lru_cache(maxsize=None)
def _get_wc_ecoom_map_lower():
    df_ecoom_category = pd.read_excel(os.path.join(BASE_DIR, 'configs/ecoom_category_20190806.xlsx'))
    wc_ecoom_map = {row['WC']: row['ECOOM'] for i, row in df_ecoom_category.iterrows()}
    wc_ecoom_map_lower = {
        k.lower().strip(): v.lower().strip()
        for k, v in wc_ecoom_map.items()
        if k and str(k) != 'nan'
    }
    return wc_ecoom_map_lower


def parse_ecoom_categorys(row, field='WC'):
    wc_ecoom_map_lower = _get_wc_ecoom_map_lower()
    wcs = str(row.get(field, '')).split(';')
    ecooms = []
    for wc in wcs:
        wc = wc.strip().lower()
        wc1 = wc.replace('&', 'and')
        if wc in wc_ecoom_map_lower:
            ecooms.append(wc_ecoom_map_lower[wc])
        elif wc1 in wc_ecoom_map_lower:
            ecooms.append(wc_ecoom_map_lower[wc1])
        else:
            ecooms.append(wc)
    return ';'.join(ecooms).lower()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import os
import re
import json
import asyncio
import datetime
import concurrent
from pathlib import Path
from collections import Counter
from pyquery import PyQuery

FIELDS = [
    'SrcDatabase',
    'Title',
    'Author',
    'Organ',
    'Source',
    'Keyword',
    'Summary',
    'PubTime',
    'FirstDuty',
    'Fund',
    'Year',
    'Volume',
    'Period',
    'PageCount',
    'CLC',
]


def parse_txt_file(file_path):
    """ 解析自定义导出文件
    """
    with open(file_path) as f:
        content = f.read()

    articles = {}
    currentIndex = 0
    currentField = None
    for line in content.split('\n'):
        if line.startswith('SrcDatabase-'):
            currentField = 'SrcDatabase'
            content = line[(line.index(':') + 1):].strip()
            article = articles.setdefault(currentIndex, {})
            if article:
                currentIndex += 1
                article = articles.setdefault(currentIndex, {})
                article[currentField] = article.get(currentField, '') + content
            else:
                article[currentField] = article.get(currentField, '') + content
        elif [i for i in FIELDS if line.startswith(i + '-')]:
            currentField = line.split('-')[0]
            content = line[(line.index(':') + 1):].strip()
            article = articles.setdefault(currentIndex, {})
            article[currentField] = article.get(currentField, '') + content
        else:
            content = line
            article = articles.setdefault(currentIndex, {})
            article[currentField] = article.get(currentField, '') + content
    return articles.values()


def read_text_format_dir(from_dir):
    for file in Path(from_dir).glob('**/*.txt'):
        yield from parse_txt_file(file)


def _parse_list_date(list_date):
    try:
        list_date = list_date.split()[0]
        tokens = list_date.split('-')
        y = int(tokens[0])
        m = int(tokens[1])
        d = int(tokens[2])
        return datetime.date(y, m, d).strftime(r'%Y-%m-%d')
    except Exception:
        return None


def read_spider_format(file_path):
    print('file_path', file_path)
    base_dir = os.path.dirname(file_path)
    files = os.listdir(base_dir)
    txt_file = [i for i in files if i.endswith('.txt')][0]
    htmls_file = [i for i in files if i.endswith('.json')][0]
    items = parse_txt_file(os.path.join(base_dir, txt_file))

    with open(os.path.join(base_dir, htmls_file)) as f:
        htmls = json.load(f)['htmls']

    list_items = []
    for html in htmls:
        for row in PyQuery(html).find('tr'):
            pq_row = PyQuery(row)
            list_name = pq_row.find('.name a').text().strip()
            if not list_name:
                continue
            list_marktip = pq_row.find('.name .marktip').text().strip()
            list_author = pq_row.find('.author').text().strip()
            list_source = pq_row.find('.source').text().strip()
            list_date = pq_row.find('.date').text().strip()
            list_data = pq_row.find('.data').text().strip()

            try:
                list_quote = int(pq_row.find('.quote').text().strip() or 0)
                list_download = int(pq_row.find('.download').text().strip() or 0)
            except (ValueError, TypeError):
                continue
            # print(
            #     f'list_name={list_name} list_marktip={list_marktip} list_author={list_author}'
            #     f'list_source={list_source} list_date={list_date} list_data={list_data}'
            #     f'list_quote={list_quote} list_download={list_download}'
            # )

            icon_collect = pq_row.find('.icon-collect')[0]
            list_dbname = icon_collect.attrib['data-dbname']
            list_filename = icon_collect.attrib['data-filename']
            list_id = f'dbname={list_dbname}&filename={list_filename}'

            list_date_format = _parse_list_date(list_date)
            if not list_date_format:
                print('list_date_format error:', list_date, list_date_format)

            list_item = dict(
                list_name=list_name,
                list_marktip=list_marktip,
                list_author=list_author,
                list_source=list_source,
                list_date=list_date,
                list_date_format=_parse_list_date(list_date),
                list_data=list_data,
                list_quote=list_quote,
                list_download=list_download,
                list_dbname=list_dbname,
                list_filename=list_filename,
                list_id=list_id,
            )
            list_items.append(list_item)

    if len(items) != len(list_items):
        print('len(items)', len(items))
        print('len(list_items)', len(list_items))
        print('error: list items length error', '=' * 50)
        # raise ValueError('list items length error')
        yield from iter([])
    for item, list_item in zip(items, list_items):
        item.update(list_item)

    yield from iter(items)


def read_spider_format_dir(from_dir):
    for file in Path(from_dir).glob('**/end'):
        yield from read_spider_format(file)


async def read_spider_format_dir_parallel(from_dir, callback, *args):
    """ parallel read
    """
    loop = asyncio.get_running_loop()
    futures = []
    with concurrent.futures.ProcessPoolExecutor() as pool:
        for file_path in Path(from_dir).glob('**/end'):
            future = loop.run_in_executor(pool, callback, file_path, *args)
            futures.append(future)
    return await asyncio.gather(*futures, return_exceptions=False)


def collect_keywords(items, keyword_field='Keyword', year_field='Year', keyword_replace_map=None, top_size=50):
    """ 搜集关键词
    """

    keyword_replace_map = keyword_replace_map or {}
    keywords = []
    keywords_map = {}
    tokens_list = []
    for item in items:
        keyword = item.get(keyword_field, '') or ''
        year = item.get(year_field, '') or ''
        if str(keyword) == 'nan' or str(year) == 'nan' or len(str(int(year))) != 4:
            continue
        tokens = re.split(r'[,;，]', keyword)
        tokens = list(set([keyword_replace_map.get(i.strip(), i.strip()) for i in tokens if i and i.strip()]))
        keywords.extend(tokens)
        keywords_map.setdefault(year, []).extend(tokens)
        tokens_list.append((year, tokens))
    counter = Counter(keywords)
    counter_map = {k: Counter(v) for k, v in keywords_map.items()}

    top_n = [k for k, v in counter.most_common(top_size)]
    print(top_n)
    years_items = []
    years_items_flat = []
    for year, year_keywords in keywords_map.items():
        years_items_flat.extend([dict(year=year, keyword=k) for k in year_keywords if k in top_n])
        for keyword in top_n:
            if keyword in year_keywords:
                years_items.append([f'{int(year)}', year_keywords.count(keyword), keyword])
    print(years_items)

    corrs = []
    for keyword1 in top_n:
        print(keyword1 + ',', end='')
        for index, keyword2 in enumerate(top_n):
            count = len([True for year, tokens in tokens_list if keyword1 in tokens and keyword2 in tokens])
            corrs.extend([
                dict(year=year, keyword1=keyword1, keyword2=keyword2)
                for year, tokens in tokens_list if keyword1 in tokens and keyword2 in tokens
            ])
            if (index + 1) == top_size:
                print(str(count), end='')
            else:
                print(str(count) + ',', end='')
        print('')

    return counter, counter_map, years_items_flat, corrs



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scilib',
 'scilib.antv',
 'scilib.cnki',
 'scilib.db',
 'scilib.gender',
 'scilib.gender.agefromname_pypi',
 'scilib.gender.api',
 'scilib.gender.api_gender',
 'scilib.gender.api_genderize',
 'scilib.gender.api_namsor',
 'scilib.gender.benchmark',
 'scilib.gender.chinese_naive_bayes',
 'scilib.gender.gender_computer',
 'scilib.gender.gender_detector',
 'scilib.gender.gender_guesser_pypi',
 'scilib.gender.gender_predictor',
 'scilib.gender.gender_r',
 'scilib.gender.genderizer_pypi',
 'scilib.gender.go_gender',
 'scilib.gender.imdb_wiki_dataset',
 'scilib.gender.tools',
 'scilib.iolib',
 'scilib.scripts',
 'scilib.stata',
 'scilib.tests',
 'scilib.tests.test_wos',
 'scilib.wos']

package_data = \
{'': ['*'],
 'scilib.tests.test_wos': ['fixtures/*'],
 'scilib.wos': ['configs/*']}

install_requires = \
['Unidecode>=1.2.0,<2.0.0',
 'agefromname>=0.0.8,<0.0.9',
 'aiohttp>=3.7.4,<4.0.0',
 'gender-guesser==0.4.0',
 'genderizer>=0.1.2,<0.2.0',
 'jupyter>=1.0.0,<2.0.0',
 'jupyterlab>=3.0.16,<4.0.0',
 'libs>=0.0.10,<0.0.11',
 'matplotlib>=3.2.1,<4.0.0',
 'naiveBayesClassifier>=0.1.3,<0.2.0',
 'nameparser>=1.0.6,<2.0.0',
 'nltk>=3.5,<4.0',
 'numpy>=1.18.2,<2.0.0',
 'openpyxl>=3.0.3,<4.0.0',
 'orjson>=3.2.1,<4.0.0',
 'pandas>=1.0.3,<2.0.0',
 'pypinyin>=0.38.1,<0.39.0',
 'pyquery>=1.4.3,<2.0.0',
 'redis>=3.5.3,<4.0.0',
 'requests>=2.23.0,<3.0.0',
 'scipy>=1.4.1,<2.0.0',
 'tabulate>=0.8.9,<0.9.0',
 'xlrd>=1.2.0,<2.0.0']

entry_points = \
{'console_scripts': ['scilib-cnki-import = scilib.scripts.cnki_import:run',
                     'scilib-db-trans = scilib.scripts.db_trans:run',
                     'scilib-gender-benchmark = '
                     'scilib.gender.benchmark.benchmark:run',
                     'scilib-wos-import = scilib.scripts.wos_import:run']}

setup_kwargs = {
    'name': 'scilib',
    'version': '0.0.14',
    'description': 'scilib',
    'long_description': '\n# scilib\n\n[![Github](https://github.com/phyng/scilib/workflows/test/badge.svg)](https://github.com/phyng/scilib/actions) [![Pypi](https://img.shields.io/pypi/v/scilib.svg?style=flat&label=PyPI)](https://pypi.org/project/scilib/)\n\n## documentation\n\nhttps://phyng.com/scilib/\n\n## install\n\n```bash\n# use pip\npip install scilib\n\n# or use poetry\npoetry add scilib\n```\n\n## usage\n\n### import wos data to ElasticSearch\n\n```bash\nenv ES_API=http://localhost:9205 scilib-wos-import --from /path/to/wos_data/ --to es --index wos\n```\n\n## test\n\n```bash\nnpm test\nnpm test_coverage\n```\n',
    'author': 'phyng',
    'author_email': 'phyngk@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/phyng/scilib',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# coding: utf-8

from .genderComputer.genderComputer import GenderComputer


def batch_classify(names):
    gc = GenderComputer()

    results = []
    for name in names:
        result = gc.resolveGender(name, None)
        if result in ['male']:
            results.append('male')
        elif result in ['female']:
            results.append('female')
        else:
            results.append('unknown')
    return results


def test():
    results = batch_classify(["tom", "lisa", "bob", "test10086"])
    print(results)


if __name__ == '__main__':
    test()



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division


def parse_orcid(text):
    """ parse OI field
    """
    if not text or str(text) == 'nan':
        return []
    state = 'NAME'  # NAME | ORCID
    name = ''
    orcid = ''

    results = []
    for c in text:
        if state == 'NAME':
            if c == '/':
                state = 'ORCID'
                continue
            elif name == '' and c in [' ', ';']:
                continue
            else:
                name += c
        elif state == 'ORCID':
            if len(orcid) == 19:
                results.append((name, orcid))
                state = 'NAME'
                name = ''
                orcid = ''
                continue
            elif c in ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', 'X']:
                orcid += c
            else:
                state = 'NAME'
                name += c
                continue
        else:
            raise ValueError(state)

    if name and orcid:
        results.append((name, orcid))
    return results


def parse_orcid_info(text, *, hmt=True):
    results = parse_orcid(text)
    infos = []
    for name, orcid in results:
        infos.append(dict(
            name=name,
            orcid=orcid,
        ))
    return infos



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

from .genderPredictor import genderPredictor


def batch_classify(names):
    gp = genderPredictor()
    gp.trainAndTest()

    first_names = [i.split()[0] for i in names]
    results = []
    for first_name in first_names:
        result = gp.classify(first_name)
        if result == 'M':
            results.append('male')
        elif result == 'F':
            results.append('female')
        else:
            results.append('unknown')
    return results



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function, division

import csv
import gzip
import json
from .es import index_or_update_rows

""" Data Trans

- pgcsv: PostgreSQL export csn with headers
- items
- es
- ndjson
- ndjsongz
"""


def pgcsv_to_items(csv_path):
    with open(csv_path, 'r') as f:
        reader = csv.reader(f)
        headers = next(reader, None)
        items = []
        for row in reader:
            item = {h: v for h, v in zip(headers, row)}
            items.append(item)
    return items


def items_to_ndjsongz(items, file_path):
    with gzip.open(file_path, 'wb') as f:
        for item in items:
            line = json.dumps(item, ensure_ascii=False) + '\n'
            f.write(line.encode('utf-8'))


def ndjsongz_to_items(file_path):
    with gzip.open(file_path, 'rb') as f:
        for line in f:
            if line:
                yield json.loads(line)


def items_to_es(items, es_index, es_pk):
    index_or_update_rows(items, index=es_index, pk=es_pk)


def pgcsv_to_es(csv_path, es_index, es_pk):
    items = pgcsv_to_items(csv_path)
    items_to_es(items, es_index, es_pk)


def ndjsongz_to_es(file_path, es_index, es_pk):
    items = ndjsongz_to_items(file_path)
    items_to_es(items, es_index, es_pk)



# coding: utf-8

from __future__ import unicode_literals, absolute_import, print_function

import itertools


def uniqify(sequence, key=None) -> iter:
    """ uniqify

    Return uniqify sequence
    """
    seen = set()
    add = seen.add
    for x in sequence:
        value = key(x) if key else x
        if value in seen:
            continue
        add(value)
        yield x


def chunks(iterable, size=10):
    """ chunks

    Split list into multiple fixed size chunks
    """
    it = iter(iterable)
    item = list(itertools.islice(it, size))
    while item:
        yield item
        item = list(itertools.islice(it, size))


def isplit(lst, count=10):
    """ isplit

    Split list into n list
    """
    return (list(lst[i::count]) for i in range(count))
